from gcs_migration.models import GuestCollection
from gcs_migration.typing.auth_documents import IdentityDocumentDict

import pytest
from tests.decorators import test_connector

from gcs_migration.check.guest_collection import (
    GuestCollectionCheck,
    GC01,
    GC02,
    GC03,
    GC04,
    GC05,
    GC06,
)


@pytest.mark.no_connector_only
@test_connector(connector="file")
def test_check_gc01(guest_collection: GuestCollection, mock_get_auth_api: None) -> None:
    gc01 = GC01()

    assert not gc01.is_ok(None)
    assert gc01.is_ok(guest_collection)

    checker = GuestCollectionCheck()
    res = checker.check(None)
    assert list(filter(lambda x: isinstance(x, GC01), res)) != []

    res = checker.check(guest_collection)
    assert list(filter(lambda x: isinstance(x, GC01), res)) == []


@pytest.mark.no_connector_only
@test_connector(connector="file")
def test_check_gc02(guest_collection: GuestCollection, mock_get_auth_api: None) -> None:
    gc02 = GC02()
    checker = GuestCollectionCheck()

    assert gc02.is_ok(None)
    assert gc02.is_ok(guest_collection)

    guest_collection.owner_id = None

    res = checker.check(None)
    assert list(filter(lambda x: isinstance(x, GC02), res)) == []

    res = checker.check(guest_collection)
    assert list(filter(lambda x: isinstance(x, GC02), res)) != []


@pytest.mark.no_connector_only
@test_connector(connector="file")
def test_check_gc03(guest_collection: GuestCollection, mock_get_auth_api: None) -> None:
    gc03 = GC03()
    checker = GuestCollectionCheck()

    assert gc03.is_ok(None)
    assert gc03.is_ok(guest_collection)

    guest_collection.owner_id = "not-a-uuid"

    res = checker.check(None)
    assert list(filter(lambda x: isinstance(x, GC03), res)) == []

    res = checker.check(guest_collection)
    assert list(filter(lambda x: isinstance(x, GC03), res)) != []


@pytest.mark.no_connector_only
@test_connector(connector="file")
def test_check_gc04(
    guest_collection: GuestCollection,
    mock_get_auth_api: None,
    unknown_identity_id: str,
    closed_idp_identity_document: IdentityDocumentDict,
    email_idp_identity_document: IdentityDocumentDict,
) -> None:
    gc04 = GC04()
    checker = GuestCollectionCheck()

    assert gc04.is_ok(None)
    assert gc04.is_ok(guest_collection)

    guest_collection.owner_id = unknown_identity_id

    res = checker.check(None)
    assert list(filter(lambda x: isinstance(x, GC04), res)) == []

    res = checker.check(guest_collection)
    assert list(filter(lambda x: isinstance(x, GC04), res)) != []

    guest_collection.owner_id = closed_idp_identity_document["id"]
    res = checker.check(guest_collection)
    assert list(filter(lambda x: isinstance(x, GC04), res)) != []

    guest_collection.owner_id = email_idp_identity_document["id"]
    res = checker.check(guest_collection)
    assert list(filter(lambda x: isinstance(x, GC04), res)) != []


@pytest.mark.no_connector_only
@test_connector(connector="file")
def test_check_gc05(guest_collection: GuestCollection, mock_get_auth_api: None) -> None:
    gc05 = GC05()
    checker = GuestCollectionCheck()

    assert gc05.is_ok(None)
    assert gc05.is_ok(guest_collection)

    guest_collection.credential = None

    res = checker.check(None)
    assert list(filter(lambda x: isinstance(x, GC05), res)) == []

    res = checker.check(guest_collection)
    assert list(filter(lambda x: isinstance(x, GC05), res)) != []


@pytest.mark.no_connector_only
@test_connector(connector="file")
def test_check_gc06(guest_collection: GuestCollection, mock_get_auth_api: None) -> None:
    gc06 = GC06()
    checker = GuestCollectionCheck()

    guest_collection.host_path = None

    assert gc06.is_ok(None)
    assert not gc06.is_ok(guest_collection)

    res = checker.check(None)
    assert list(filter(lambda x: isinstance(x, GC06), res)) == []

    res = checker.check(guest_collection)
    assert list(filter(lambda x: isinstance(x, GC06), res)) != []

    guest_collection.host_path = ""

    assert gc06.is_ok(guest_collection)
    res = checker.check(guest_collection)
    assert list(filter(lambda x: isinstance(x, GC06), res)) == []

    guest_collection.base_path = "/foo"

    assert gc06.is_ok(guest_collection)
    res = checker.check(guest_collection)
    assert list(filter(lambda x: isinstance(x, GC06), res)) == []
