from typing import List

import pytest

from tests.fixtures import *  # noqa: F401, F403


def pytest_collection_modifyitems(
    session: pytest.Session, config: pytest.Config, items: List[pytest.Item]
) -> None:
    # Filter out connector-specific variations that don't match the current
    # connector value
    remove_items = set()
    for item in items:
        # If marked with no_connector_only, skip this if *any* connector_only
        # marks are seen
        no_connector_only_marker = item.get_closest_marker("no_connector_only")
        if no_connector_only_marker:
            connector_only_marker = item.get_closest_marker("connector_only")
            if connector_only_marker:
                remove_items.add(item)
                continue

        # If marked for a particular connector, ignore if there are any
        # single connector parameters for other connectors
        connector_marker = item.get_closest_marker("connector")
        if connector_marker:
            connector = connector_marker.args[0]
            for connector_only_marker in item.iter_markers("connector_only"):
                connector_only = connector_only_marker.args[0]
                if connector != connector_only:
                    remove_items.add(item)
                    break
    for item in remove_items:
        items.remove(item)
